%% This class provides a house with a variable solar
%% panel size and a consumption for the whole year. These
%% blocks are connected to a @GridController 

classdef House < handle
   
    properties (Constant)
       %% Path to Block in library
       SOURCE = 'smartlib/House';
        
       IN_SIZE                = '/1';
       IN_BATTERY             = '/2';
       IN_YEAR_KWH            = '/3';
       IN_EEGSTARTINGMONTH    = '/4';
       IN_REMAININGEEGHOURS   = '/5';
       
       OUT_KWH_CONS    = '/1';
       OUT_KWH_PROD    = '/2';
       OUT_MERGED      = '/3';
       OUT_SPLIT       = '/4';
       OUT_GRADIENT    = '/5';
       
       
    end
    
    properties
        name %% Name of Block in Simulink
        
        size %% Size of the solar panel
        consum %% Consumption per annum
        
        remainingEegHours
        eegStartingMonth
    end
    
    methods

        %% Creates a new house in the Simulink file with the given
        %% solar panel size and the consumption for the whole year
        function obj = House(system, consumption, solarSize, eegStartingMonth, remainingEegHours)
            obj.name = ['House_' int2str(obj.getCount(1))];
            
            obj.size = solarSize;
            obj.consum = consumption;
            
            obj.eegStartingMonth = eegStartingMonth;
            obj.remainingEegHours = remainingEegHours;
            
            % Adds a CONSTANT block which is used for the solar panel size
            add_block(Simulink.CONSTANT, [system.name '/' obj.name '_Solar_Panel_Size_(m^2)']);
            set_param([system.name '/' obj.name '_Solar_Panel_Size_(m^2)'], 'Value', obj.size{1});
            
            % Adds a CONSTANT block which is used for the demand throughout the year
            add_block(Simulink.CONSTANT, [system.name '/' obj.name '_Yearly_Power_Demand_(kWh)']);
            set_param([system.name '/' obj.name '_Yearly_Power_Demand_(kWh)'], 'Value', obj.consum{1});
            
            % Adds a CONSTANT block which is used for the year of consumption start
            add_block(Simulink.CONSTANT, [system.name '/' obj.name '_First_Month_of_PV_Production']);
            set_param([system.name '/' obj.name '_First_Month_of_PV_Production'], 'Value', num2str(obj.eegStartingMonth));
            
            % Adds a CONSTANT block which is used for the month of consumption start
            add_block(Simulink.CONSTANT, [system.name '/' obj.name '_Remaining_Feed-In_Tariff_Hours']);
            set_param([system.name '/' obj.name '_Remaining_Feed-In_Tariff_Hours'], 'Value', num2str(obj.remainingEegHours));
            
            % Add blocks for saving outputs to workspace
            add_block(Simulink.TO_WORKSPACE, [system.name '/' obj.name '_Merged_Amortisation']);
            set_param([system.name '/' obj.name '_Merged_Amortisation'], 'VariableName', [obj.name '_merged_amortisation']);
            
            add_block(Simulink.TO_WORKSPACE, [system.name '/' obj.name '_Split_Amortisation']);
            set_param([system.name '/' obj.name '_Split_Amortisation'], 'VariableName', [obj.name '_split_amortisation']);
            
            add_block(Simulink.TO_WORKSPACE, [system.name '/' obj.name '_Grid_Sale_Gradient']);
            set_param([system.name '/' obj.name '_Grid_Sale_Gradient'], 'VariableName', [obj.name '_grid_sale_gradient']);
            
            % Adds a House_F1 block to the current file
            add_block(obj.SOURCE, [system.name '/' obj.name]);
            
            % Connects in- and outputs to house
            add_line(system.name, [obj.name '_Solar_Panel_Size_(m^2)/1'], [obj.name obj.IN_SIZE]);
            add_line(system.name, [obj.name '_Yearly_Power_Demand_(kWh)/1'], [obj.name obj.IN_YEAR_KWH]);
            
            add_line(system.name, [obj.name '_First_Month_of_PV_Production/1'], [obj.name obj.IN_EEGSTARTINGMONTH]);
            add_line(system.name, [obj.name '_Remaining_Feed-In_Tariff_Hours/1'], [obj.name obj.IN_REMAININGEEGHOURS]);
            
            add_line(system.name, [obj.name obj.OUT_MERGED], [obj.name '_Merged_Amortisation/1']);
            add_line(system.name, [obj.name obj.OUT_SPLIT], [obj.name '_Split_Amortisation/1']);
            add_line(system.name, [obj.name obj.OUT_GRADIENT], [obj.name '_Grid_Sale_Gradient/1']);
        end
         
        %% Sets the consumption of the year for the house
        function setConsumption(obj, value)
            obj.consum = value;
            set_param([system.name '/' obj.name '_demand'], 'Value', num2str(obj.consum));
        end
        
    end
    
     methods (Static)

         %% Returns the current number of Houses
         %% which are generated by the script
         function f = getCount(value)
             persistent anz;
             
             if isempty(anz), anz = 0; end
             if nargin, anz = anz + value; end

             f = anz;
         end
      
   end
    
end