%% This class creates a Simulink file for the
%% simulation of a system. It is used to setup the MicroGrid,
%% to add all important workspace variables and generate
%% the output of the system.

classdef System < handle
   
    properties (Constant)
        % Path to Block in our custom library
        SOURCE = 'smartlib/Microgridcontroller';
        
        % Strings for model variables
        NUMBER_OF_HOUSES           = 'numberOfHouses';
        BATTERY_COST_KWH           = 'batteryCostPerKwh';
        BATTERY_SIZE               = 'batterySize';
        ONE_TIME_COST              = 'oneTimeCost';
        COST_UTILITY_KWH           = 'costPerKwhFromUtility';
        CHANGE_COST_UTILITY_KWH    = 'annualChangeOfCostPerKwhFromUtility';
        SOLAR_PANEL_EFFICIENCY     = 'solarPanelEfficiency';
        BUY_FROM_BATTERY_KWH       = 'costPerKwhBoughtFromBattery';
        CONSUMPTION_BATTERY_KWH    = 'compensationPerKwhSoldToBattery';
        HOURS_PER_YEAR             = 24 * 365;
    end
    
    properties
        name %% Holds the filename of the simulink simulation file
    end
    
    methods

        %% Creates a new Simulink file which
        %% is used for the whole simulation
        function obj = System(name)
            obj.name = name;
            
            new_system(name);
            open_system(name);
        end
        
        %% Sets up a a complete simulation of a
        %% SmartGrid in the current Simulink file
        function setup(obj, houses, bcKWH, bs, otc, cuKWH, ccuKWH, bid, spe, bfbKWH, cbKWH, sd)
            
            % Check to see if library is likely to be up to date
            basefile = 'SmartHomes.slx';
            try 
                % Get dates of last modification
                basefiletime = dir(basefile);
                libfiletime = dir('lib/smartlib.slx');
                
                % See if main model was updated after library
                if libfiletime.datenum < basefiletime.datenum
                    warning('Library might be out of date!');
                end
            catch
                warning(['Can''t assert age of library. Probably missing or can''t access ' basefile '. (Incorrect working directory?)']);
            end
            
            % Create the GridController for the simulation
            grid = GridController(obj, (numel(houses) / 3));
            houses = transpose(houses); % Transpose for easier use
            
            batteryInstallDate = datetime(bid, 'InputFormat', 'MM-yyyy');
            
            % Houses matrix looks like this (Index)
            %   -----------------------------------------------~~~
            %   |  (1)  SolarSize 1    |  (4)  SolarSize 2    |
            %   |  (2)  Consume Year 1 |  (5)  Consume Year 2 |
            %   |  (3)  Panel Date 1   |  (6)  Panel Date 2   |
            %   -----------------------------------------------~~~
            
            % For each house...
            for idx = 1 : 3 : numel(houses)
                solarSize = houses(idx);
                consume = houses(idx + 1);
                pvInstallDate = datetime(houses(idx + 2), 'InputFormat', 'MM-yyyy');
                
                % Calculate the number of months that have passed since
                % January 2004 in order to figure out the applicable
                % feed-in tariff for that year and month
                eegStartingMonth = (pvInstallDate.Year - 2004) * 12 + (pvInstallDate.Month - 1);
                
                % Calculate remaining time for which the house receives the
                % feed-in tariff
                
                B = batteryInstallDate;
                P = pvInstallDate;
                
                elapsedHours = 0;
                
                while (P.Month ~= B.Month || P.Year ~= B.Year)
                    elapsedHours = elapsedHours + obj.hoursInMonth(P.Month);
                    if (P.Month == 12)
                        P.Month = 1;
                        P.Year  = P.Year + 1;
                    else
                        P.Month = P.Month + 1;
                    end
                end
                
                % Feed-in tariff is paid for 20 years from the first day
                % the PV installation started producing electricity
                remainingEegHours = 20 * 8760 - elapsedHours;
                
                h = House(obj, consume, solarSize, eegStartingMonth, remainingEegHours);
                grid.connect(h); % Connect house to grid
            end
            
            obj.addVariable(obj.BATTERY_COST_KWH, str2double(bcKWH));
            obj.addVariable(obj.BATTERY_SIZE, str2double(bs));
            obj.addVariable(obj.ONE_TIME_COST, str2double(otc));
            obj.addVariable(obj.COST_UTILITY_KWH, str2double(cuKWH));
            obj.addVariable(obj.CHANGE_COST_UTILITY_KWH, str2double(ccuKWH));
            obj.addVariable(obj.SOLAR_PANEL_EFFICIENCY, str2double(spe));
            obj.addVariable(obj.BUY_FROM_BATTERY_KWH, str2double(bfbKWH));
            obj.addVariable(obj.CONSUMPTION_BATTERY_KWH, str2double(cbKWH));
            obj.addVariable(obj.NUMBER_OF_HOUSES, numel(houses) / 3);
            
            % Set the stop time for the simulation
            set_param(obj.name, 'StopTime', num2str(str2double(sd) * obj.HOURS_PER_YEAR));
        end
        
        %% Cleans up the connections between all blocks
        %% in the current Simulink file for better readability.
        %% Requires "Auto Layout Tool" from
        %% https://www.mathworks.com/matlabcentral/fileexchange/51228-auto-layout-tool
        function optimize(obj)
            AutoLayout(obj.name);
        end
        
        %% Returns number of hours per given month.
        %% Does NOT care about leap years!
        function returnValue = hoursInMonth(~, numberOfMonth)
            
            if (numberOfMonth ==  1 || ...
                numberOfMonth ==  3 || ...
                numberOfMonth ==  5 || ...
                numberOfMonth ==  7 || ...
                numberOfMonth ==  8 || ...
                numberOfMonth == 10 || ...
                numberOfMonth == 12)
                returnValue = 744;
            elseif (numberOfMonth ==  4 || ...
                    numberOfMonth ==  6 || ...
                    numberOfMonth ==  9 || ...
                    numberOfMonth ==  11)
                returnValue = 720;
            else
                returnValue = 672;
            end
            
        end

        %% Starts the simulation of the SimuLink file
        function out = simulate(obj)
            out = sim(obj.name, '');
        end
        
        %% Adds a variable with its value to the Simulink Workspace
        function addVariable(obj, name, value)
            ws = get_param(obj.name, 'modelworkspace');
            ws.assignin(name, value);
        end
        
    end
    
end